@@ -21,13 +21,12 @@
  */
 
 #include <IOKit/audio/IOAudioDevice.h>
-#include <IOKit/audio/IOAudioDMAEngine.h>
+#include <IOKit/audio/IOAudioEngine.h>
 #include <IOKit/audio/IOAudioPort.h>
 #include <IOKit/audio/IOAudioTypes.h>
 #include <IOKit/audio/IOAudioDefines.h>
 #include <IOKit/audio/IOAudioLevelControl.h>
-#include <IOKit/audio/IOAudioMuteControl.h>
-#include <IOKit/audio/IOAudioManager.h>
+#include <IOKit/audio/IOAudioToggleControl.h>
 #include <IOKit/IOWorkLoop.h>
 #include <IOKit/IOCommandGate.h>
 #include <IOKit/IOTimerEventSource.h>
@@ -52,36 +51,164 @@ protected:
 
 OSDefineMetaClassAndStructors(IOAudioTimerEvent, OSObject)
 
-class IOAudioDMAEngineEntry : public OSObject
+class IOAudioEngineEntry : public OSObject
 {
     friend class IOAudioDevice;
 
-    OSDeclareDefaultStructors(IOAudioDMAEngineEntry);
+    OSDeclareDefaultStructors(IOAudioEngineEntry);
 
 protected:
-    IOAudioDMAEngine *audioDMAEngine;
-    bool shouldStopDMAEngine;
+    IOAudioEngine *audioEngine;
+    bool shouldStopAudioEngine;
 };
 
-OSDefineMetaClassAndStructors(IOAudioDMAEngineEntry, OSObject)
+OSDefineMetaClassAndStructors(IOAudioEngineEntry, OSObject)
 
 #define super IOService
 OSDefineMetaClassAndStructors(IOAudioDevice, IOService)
+OSMetaClassDefineReservedUsed(IOAudioDevice, 0);
+OSMetaClassDefineReservedUsed(IOAudioDevice, 1);
+OSMetaClassDefineReservedUsed(IOAudioDevice, 2);
 
+OSMetaClassDefineReservedUnused(IOAudioDevice, 3);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 4);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 5);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 6);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 7);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 8);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 9);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 10);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 11);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 12);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 13);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 14);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 15);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 16);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 17);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 18);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 19);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 20);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 21);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 22);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 23);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 24);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 25);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 26);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 27);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 28);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 29);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 30);
+OSMetaClassDefineReservedUnused(IOAudioDevice, 31);
+
+// New code added here
+void IOAudioDevice::setDeviceTransportType(const UInt32 transportType)
+{
+    if (transportType) {
+        setProperty(kIOAudioDeviceTransportTypeKey, transportType, 32);
+    }
+}
+
+// This needs to be overridden by driver if it wants to know about power manager changes.
+// If overridden, be sure to still call super::setAggressiveness() so we can call our parent.
+IOReturn IOAudioDevice::setAggressiveness(unsigned long type, unsigned long newLevel)
+{
+	return super::setAggressiveness(type, newLevel);
+}
+
+void IOAudioDevice::setIdleAudioSleepTime(unsigned long long sleepDelay)
+{
+	assert(reserved);
+
+#ifdef DEBUG_CALLS
+	IOLog ("IOAudioDevice[%p]::setIdleAudioSleepTime: sleepDelay = %lx%lx\n", this, sleepDelay >> 32, sleepDelay);
+#endif
+	if (reserved->idleSleepDelayTime != sleepDelay) {
+		reserved->idleSleepDelayTime = sleepDelay;
+		scheduleIdleAudioSleep();
+	}
+
+	if (reserved->idleSleepDelayTime == kNoIdleAudioPowerDown) {
+		if (reserved->idleTimer) {
+			reserved->idleTimer->cancelTimeout();
+		}
+	}
+}
+
+// Set up a timer to power down the hardware if we haven't used it in a while.
+void IOAudioDevice::scheduleIdleAudioSleep(void)
+{
+    AbsoluteTime				fireTime;
+    UInt64						nanos;
+
+	assert(reserved);
+
+#ifdef DEBUG_CALLS
+	IOLog ("IOAudioDevice[%p]::scheduleIdleAudioSleep: idleSleepDelayTime = %lx%lx\n", this, reserved->idleSleepDelayTime >> 32, reserved->idleSleepDelayTime);
+#endif
+	if (reserved->idleSleepDelayTime == 0) {
+		// For backwards compatibility, or drivers that don't care, tell them about idle right away.
+		initiatePowerStateChange();
+	}
+
+	if (reserved->idleTimer && reserved->idleSleepDelayTime != kNoIdleAudioPowerDown) {
+		// If the driver wants to know about idle sleep after a specific amount of time, then set the timer to tell them at that time.
+		// If idleSleepDelayTime == 0xffffffff then don't ever tell the driver about going idle
+		clock_get_uptime(&fireTime);
+		absolutetime_to_nanoseconds(fireTime, &nanos);
+		nanos += reserved->idleSleepDelayTime;
+		nanoseconds_to_absolutetime(nanos, &fireTime);
+		reserved->idleTimer->wakeAtTime(fireTime);		// will call idleAudioSleepHandlerTimer
+	}
+
+	return;
+}
+
+void IOAudioDevice::idleAudioSleepHandlerTimer(OSObject *owner, IOTimerEventSource *sender)
+{
+	IOAudioDevice *				audioDevice;
+
+	audioDevice = OSDynamicCast(IOAudioDevice, owner);
+	assert(audioDevice);
+
+#ifdef DEBUG_CALLS
+	IOLog ("IOAudioDevice[%p]idleAudioSleepHandlerTimer: pendingPowerState = %ld, idleSleepDelayTime = %lx%lx\n", this, audioDevice->pendingPowerState, audioDevice->reserved->idleSleepDelayTime >> 32, audioDevice->reserved->idleSleepDelayTime);
+#endif
+	if (audioDevice->reserved->idleSleepDelayTime != kNoIdleAudioPowerDown &&
+		audioDevice->getPendingPowerState () == kIOAudioDeviceIdle) {
+		// If we're still idle, tell the device to go idle now that the requested amount of time has elapsed.
+		audioDevice->initiatePowerStateChange();
+	}
+
+	return;
+}
+
+// Original code here...
 const IORegistryPlane *IOAudioDevice::gIOAudioPlane = 0;
 
 bool IOAudioDevice::init(OSDictionary *properties)
 {
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::init(%p)\n", this, properties);
+#endif
+
     if (!super::init(properties)) {
         return false;
     }
 
+	reserved = (ExpansionData *)IOMalloc (sizeof(struct ExpansionData));
+	if (!reserved) {
+		return false;
+	} else {
+		reserved->idleSleepDelayTime = 0;
+		reserved->idleTimer = NULL;
+	}
+
     if (!gIOAudioPlane) {
         gIOAudioPlane = IORegistryEntry::makePlane(kIOAudioPlane);
     }
 
-    audioDMAEngines = OSSet::withCapacity(2);
-    if (!audioDMAEngines) {
+    audioEngines = OSArray::withCapacity(2);
+    if (!audioEngines) {
         return false;
     }
 
@@ -95,31 +222,35 @@ bool IOAudioDevice::init(OSDictionary *properties)
         return false;
     }
     
-    wakingFromSleep = false;
     familyManagePower = true;
+    asyncPowerStateChangeInProgress = false;
+    
+    currentPowerState = kIOAudioDeviceIdle;
+    pendingPowerState = kIOAudioDeviceIdle;
+    
+    numRunningAudioEngines = 0;
     
     return true;
 }
 
 void IOAudioDevice::free()
 {
-    if (audioDMAEngines) {
-        deactivateAudioDMAEngines();
-        audioDMAEngines->release();
-        audioDMAEngines = 0;
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::free()\n", this);
+#endif
+
+    if (audioEngines) {
+        deactivateAllAudioEngines();
+        audioEngines->release();
+        audioEngines = 0;
     }
 
     if (audioPorts) {
-        deactivateAudioPorts();
+        detachAllAudioPorts();
         audioPorts->release();
         audioPorts = 0;
     }
     
-    if (masterControls) {
-        masterControls->release();
-        masterControls = 0;
-    }
-
     if (timerEvents) {
         timerEvents->release();
         timerEvents = 0;
@@ -133,7 +264,16 @@ void IOAudioDevice::free()
         timerEventSource->release();
         timerEventSource = NULL;
     }
-    
+
+	if (reserved->idleTimer) {
+		if (workLoop) {
+			workLoop->removeEventSource(reserved->idleTimer);
+		}
+
+		reserved->idleTimer->release();
+		reserved->idleTimer = NULL;
+	}
+
     if (commandGate) {
         if (workLoop) {
             workLoop->removeEventSource(commandGate);
@@ -147,12 +287,20 @@ void IOAudioDevice::free()
         workLoop->release();
         workLoop = NULL;
     }
+
+	if (reserved) {
+		IOFree (reserved, sizeof(struct ExpansionData));
+	}
     
     super::free();
 }
 
 bool IOAudioDevice::initHardware(IOService *provider)
 {
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::initHardware(%p)\n", this, provider);
+#endif
+
     return true;
 }
 
@@ -163,9 +311,22 @@ bool IOAudioDevice::start(IOService *provider)
         {1, IOPMDeviceUsable, IOPMPowerOn, IOPMPowerOn, 0, 0, 0, 0, 0, 0, 0, 0}
     };
     
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::start(%p)\n", this, provider);
+#endif
+
     if (!super::start(provider)) {
         return false;
     }
+    
+    assert(workLoop);
+    
+    commandGate = IOCommandGate::commandGate(this);
+    if (!commandGate) {
+        return false;
+    }
+    
+    workLoop->addEventSource(commandGate);
 
     if (!initHardware(provider)) {
         return false;
@@ -181,6 +342,12 @@ bool IOAudioDevice::start(IOService *provider)
             changePowerStateTo(1);
             duringStartup = false;
         }
+
+		reserved->idleTimer = IOTimerEventSource::timerEventSource(this, idleAudioSleepHandlerTimer);
+		if (!reserved->idleTimer) {
+			return false;
+		}
+		workLoop->addEventSource(reserved->idleTimer);
     }
 
     registerService();
@@ -189,8 +356,10 @@ bool IOAudioDevice::start(IOService *provider)
 }
 
 void IOAudioDevice::stop(IOService *provider)
-{
-    IOAudioManager *manager;
+{    
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::stop(%p)\n", this, provider);
+#endif
     
     if (timerEventSource) {
         if (workLoop) {
@@ -201,10 +370,19 @@ void IOAudioDevice::stop(IOService *provider)
         timerEventSource = NULL;
     }
 
-    clearTimerEvents();
+    if (reserved->idleTimer) {
+        if (workLoop) {
+            workLoop->removeEventSource(reserved->idleTimer);
+        }
 
-    deactivateAudioDMAEngines();
-    deactivateAudioPorts();
+        reserved->idleTimer->release();
+        reserved->idleTimer = NULL;
+    }
+
+    removeAllTimerEvents();
+
+    deactivateAllAudioEngines();
+    detachAllAudioPorts();
 
     if (familyManagePower) {
         PMstop();
@@ -219,11 +397,6 @@ void IOAudioDevice::stop(IOService *provider)
         commandGate = NULL;
     }
 
-    manager = IOAudioManager::sharedInstance();
-    if (manager) {
-        manager->removeAudioDevice(this);
-    }
-    
     super::stop(provider);
 }
 
@@ -237,32 +410,19 @@ IOReturn IOAudioDevice::setPowerState(unsigned long powerStateOrdinal, IOService
     IOReturn result = IOPMCannotRaisePower;
     
 #ifdef DEBUG_CALLS
-    kprintf("IOAudioDevice[%p]::setPowerState(%lu, %p)\n", this, powerStateOrdinal, device);
     IOLog("IOAudioDevice[%p]::setPowerState(%lu, %p)\n", this, powerStateOrdinal, device);
 #endif
     
     if (!duringStartup) {
         if (powerStateOrdinal >= NUM_POWER_STATES) {
             result = IOPMNoSuchState;
-        } else if (powerStateOrdinal == 0) {
+        } else {
             IOCommandGate *cg;
             
             cg = getCommandGate();
             
             if (cg) {
-                result = cg->runAction(setPowerStateSleepAction);
-            }
-            
-            wakingFromSleep = true;
-        } else if ((powerStateOrdinal == 1) && (wakingFromSleep)) {
-            IOCommandGate *cg;
-            
-            wakingFromSleep = false;
-            
-            cg = getCommandGate();
-            
-            if (cg) {
-                result = cg->runAction(setPowerStateWakeAction);
+                result = cg->runAction(setPowerStateAction, (void *)powerStateOrdinal, (void *)device);
             }
         }
     }
@@ -270,256 +430,402 @@ IOReturn IOAudioDevice::setPowerState(unsigned long powerStateOrdinal, IOService
     return result;
 }
 
-IOReturn IOAudioDevice::setPowerStateSleepAction(OSObject *owner, void *arg1, void *arg2, void *arg3, void *arg4)
+IOReturn IOAudioDevice::setPowerStateAction(OSObject *owner, void *arg1, void *arg2, void *arg3, void *arg4)
 {
     IOReturn result = kIOReturnBadArgument;
     
-#ifdef DEBUG_CALLS
-    kprintf("IOAudioDevice::setPowerStateSleepAction(%p)\n", owner);
-    IOLog("IOAudioDevice::setPowerStateSleepAction(%p)\n", owner);
-#endif
-
     if (owner) {
         IOAudioDevice *audioDevice = OSDynamicCast(IOAudioDevice, owner);
         
         if (audioDevice) {
-            audioDevice->setPowerStateSleep();
+            result = audioDevice->protectedSetPowerState((unsigned long)arg1, (IOService *)arg2);
         }
     }
     
     return result;
 }
 
-IOReturn IOAudioDevice::setPowerStateWakeAction(OSObject *owner, void *arg1, void *arg2, void *arg3, void *arg4)
+IOReturn IOAudioDevice::protectedSetPowerState(unsigned long powerStateOrdinal, IOService *device)
 {
-    IOReturn result = kIOReturnBadArgument;
-    
-#ifdef DEBUG_CALLS
-    kprintf("IOAudioDevice::setPowerStateWakeAction(%p)\n", owner);
-    IOLog("IOAudioDevice::setPowerStateWakeAction(%p)\n", owner);
-#endif
+    IOReturn result = IOPMAckImplied;
 
-    if (owner) {
-        IOAudioDevice *audioDevice = OSDynamicCast(IOAudioDevice, owner);
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::protectedSetPowerState(%lu, %p)\n", this, powerStateOrdinal, device);
+#endif
+    
+    if (asyncPowerStateChangeInProgress) {
+        waitForPendingPowerStateChange();
+    }
+    
+    if (powerStateOrdinal == 0) {	// Sleep
+        if (getPowerState() != kIOAudioDeviceSleep) {
+            pendingPowerState = kIOAudioDeviceSleep;
+
+            // Stop all audio engines
+            if (audioEngines && (numRunningAudioEngines > 0)) {
+                OSCollectionIterator *audioEngineIterator;
+                
+                audioEngineIterator = OSCollectionIterator::withCollection(audioEngines);
+                
+                if (audioEngineIterator) {
+                    IOAudioEngine *audioEngine;
+                    
+                    while (audioEngine = (IOAudioEngine *)audioEngineIterator->getNextObject()) {
+                        if (audioEngine->getState() == kIOAudioEngineRunning) {
+                            audioEngine->pauseAudioEngine();
+                        }
+                    }
+                    
+                    audioEngineIterator->release();
+                }
+            }
+        }
+    } else if (powerStateOrdinal == 1) {	// Wake
+        if (getPowerState() == kIOAudioDeviceSleep) {	// Need to change state if sleeping
+            if (numRunningAudioEngines == 0) {
+                pendingPowerState = kIOAudioDeviceIdle;
+            } else {
+                pendingPowerState = kIOAudioDeviceActive;
+            }
+        }
+    }
+    
+    if (currentPowerState != pendingPowerState) {
+        UInt32 microsecondsUntilComplete = 0;
         
-        if (audioDevice) {
-            audioDevice->setPowerStateWake();
+        result = initiatePowerStateChange(&microsecondsUntilComplete);
+        if (result == kIOReturnSuccess) {
+            result = microsecondsUntilComplete;
         }
     }
     
     return result;
 }
 
-IOReturn IOAudioDevice::setPowerStateSleep()
+void IOAudioDevice::waitForPendingPowerStateChange()
+{
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::waitForPendingPowerStateChange()\n", this);
+#endif
+
+    if (asyncPowerStateChangeInProgress) {
+        IOCommandGate *cg;
+        
+        cg = getCommandGate();
+        
+        if (cg) {
+            cg->commandSleep((void *)&asyncPowerStateChangeInProgress);
+            assert(!asyncPowerStateChangeInProgress);
+        } else {
+            IOLog("IOAudioDevice[%p]::waitForPendingPowerStateChange() - internal error - unable to get the command gate.\n", this);
+            return;
+        }
+    }
+}
+
+IOReturn IOAudioDevice::initiatePowerStateChange(UInt32 *microsecondsUntilComplete)
 {
     IOReturn result = kIOReturnSuccess;
-    
-#ifdef DEBUG_CALLS
-    kprintf("IOAudioDevice[%p]::setPowerStateSleep()\n", this);
-    IOLog("IOAudioDevice[%p]::setPowerStateSleep()\n", this);
-#endif
 
-    if (audioDMAEngines) {
-        OSCollectionIterator *dmaEngineIterator;
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::initiatePowerStateChange(%p) - current = %d - pending = %d\n", this, microsecondsUntilComplete, currentPowerState, pendingPowerState);
+#endif
+    
+    if (currentPowerState != pendingPowerState) {
+        UInt32 localMicsUntilComplete, *micsUntilComplete = NULL;
         
-        dmaEngineIterator = OSCollectionIterator::withCollection(audioDMAEngines);
+        if (microsecondsUntilComplete != NULL) {
+            micsUntilComplete = microsecondsUntilComplete;
+        } else {
+            micsUntilComplete = &localMicsUntilComplete;
+        }
         
-        if (dmaEngineIterator) {
-            IOAudioDMAEngine *audioDMAEngine;
-            
-            while (audioDMAEngine = (IOAudioDMAEngine *)dmaEngineIterator->getNextObject()) {
-                if (audioDMAEngine->getState() == kAudioDMAEngineRunning) {
-                    audioDMAEngine->stopDMAEngine();
-                }
+        *micsUntilComplete = 0;
+        
+        asyncPowerStateChangeInProgress = true;
+        
+        result = performPowerStateChange(currentPowerState, pendingPowerState, micsUntilComplete);
+        
+        if (result == kIOReturnSuccess) {
+            if (*micsUntilComplete == 0) {
+                asyncPowerStateChangeInProgress = false;
+                protectedCompletePowerStateChange();
             }
-            
-            dmaEngineIterator->release();
+        } else {
+            asyncPowerStateChangeInProgress = false;
         }
     }
     
-    result = performDeviceSleep();
-    
-    return kIOReturnSuccess;
+    return result;
 }
 
-IOReturn IOAudioDevice::setPowerStateWake()
+IOReturn IOAudioDevice::completePowerStateChange()
 {
-    IOReturn result;
+    IOReturn result = kIOReturnError;
+    IOCommandGate *cg;
     
+    cg = getCommandGate();
+    
+    if (cg) {
+        result = cg->runAction(completePowerStateChangeAction);
+    }
+    
+    return result;
+}
+
+IOReturn IOAudioDevice::completePowerStateChangeAction(OSObject *owner, void *arg1, void *arg2, void *arg3, void *arg4)
+{
+    IOReturn result = kIOReturnBadArgument;
+    
+    if (owner) {
+        IOAudioDevice *audioDevice = OSDynamicCast(IOAudioDevice, owner);
+        
+        if (audioDevice) {
+            result = audioDevice->protectedCompletePowerStateChange();
+        }
+    }
+    
+    return result;
+}
+
+IOReturn IOAudioDevice::protectedCompletePowerStateChange()
+{
+    IOReturn result = kIOReturnSuccess;
+
 #ifdef DEBUG_CALLS
-    kprintf("IOAudioDevice[%p]::setPowerStateWake()\n", this);
-    IOLog("IOAudioDevice[%p]::setPowerStateWake()\n", this);
+    IOLog("IOAudioDevice[%p]::protectedCompletePowerStateChange() - current = %d - pending = %d\n", this, currentPowerState, pendingPowerState);
 #endif
 
-    result = performDeviceWake();
-    
-    if (result == kIOReturnSuccess) {
-        clock_get_uptime(&previousTimerFire);
-        SUB_ABSOLUTETIME(&previousTimerFire, &minimumInterval);
-        
-        if (timerEvents && (timerEvents->getCount() > 0)) {
-            dispatchTimerEvents(true);
-        }
-        
-        if (audioDMAEngines) {
-            OSCollectionIterator *dmaEngineIterator;
+    if (currentPowerState != pendingPowerState) {
+        // If we're waking, we fire off the timers and resync them
+        // Then restart the audio engines that were running before the sleep
+        if (currentPowerState == kIOAudioDeviceSleep) {	
+            clock_get_uptime(&previousTimerFire);
+            SUB_ABSOLUTETIME(&previousTimerFire, &minimumInterval);
             
-            dmaEngineIterator = OSCollectionIterator::withCollection(audioDMAEngines);
+            if (timerEvents && (timerEvents->getCount() > 0)) {
+                dispatchTimerEvents(true);
+            }
             
-            if (dmaEngineIterator) {
-                IOAudioDMAEngine *audioDMAEngine;
+            if (audioEngines && (numRunningAudioEngines > 0)) {
+                OSCollectionIterator *audioEngineIterator;
                 
-                while (audioDMAEngine = (IOAudioDMAEngine *)dmaEngineIterator->getNextObject()) {
-                    if (audioDMAEngine->getState() == kAudioDMAEngineRunning) {
-                        UInt32 loopCount;
-                        // We can't reset the loop count because the HAL get's confused
-                        // Once we do stop/start notifications we can remove the code to reset the loop count
-                        loopCount = audioDMAEngine->getStatus()->fCurrentLoopCount;
-                        audioDMAEngine->resetStatusBuffer();
-                        ((IOAudioDMAEngineStatus *)audioDMAEngine->getStatus())->fCurrentLoopCount = loopCount + 1;
-                        
-                        audioDMAEngine->clearAllSampleBuffers();
-                        
-                        audioDMAEngine->startDMAEngine();
+                audioEngineIterator = OSCollectionIterator::withCollection(audioEngines);
+                
+                if (audioEngineIterator) {
+                    IOAudioEngine *audioEngine;
+                    
+                    while (audioEngine = (IOAudioEngine *)audioEngineIterator->getNextObject()) {
+                        if (audioEngine->getState() == kIOAudioEnginePaused) {
+                            audioEngine->resumeAudioEngine();
+                        }
                     }
+                        
+                    audioEngineIterator->release();
                 }
-                
-                dmaEngineIterator->release();
             }
         }
+    
+        if (asyncPowerStateChangeInProgress) {
+            IOCommandGate *cg;
+            
+            acknowledgePowerChange(this);
+            asyncPowerStateChangeInProgress = false;
+        
+            cg = getCommandGate();
+            if (cg) {
+                cg->commandWakeup((void *)&asyncPowerStateChangeInProgress);
+            }
+        }
+        
+        currentPowerState = pendingPowerState;
     }
     
-    return kIOReturnSuccess;
+    return result;
 }
 
-IOReturn IOAudioDevice::performDeviceSleep()
+IOReturn IOAudioDevice::performPowerStateChange(IOAudioDevicePowerState oldPowerState,
+                                                IOAudioDevicePowerState newPowerState,
+                                                UInt32 *microsecondsUntilComplete)
 {
     return kIOReturnSuccess;
 }
 
-IOReturn IOAudioDevice::performDeviceWake()
+IOAudioDevicePowerState IOAudioDevice::getPowerState()
 {
-    return kIOReturnSuccess;
+    return currentPowerState;
 }
 
-IOWorkLoop *IOAudioDevice::getWorkLoop()
+IOAudioDevicePowerState IOAudioDevice::getPendingPowerState()
+{
+    return pendingPowerState;
+}
+
+void IOAudioDevice::audioEngineStarting()
+{
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::audioEngineStarting() - numRunningAudioEngines = %ld\n", this, numRunningAudioEngines + 1);
+#endif
+
+    numRunningAudioEngines++;
+    
+    if (numRunningAudioEngines == 1) {	// First audio engine starting - need to be in active state
+        if (getPowerState() == kIOAudioDeviceIdle) {	// Go active
+            if (asyncPowerStateChangeInProgress) {	// Sleep if there is a transition in progress
+                waitForPendingPowerStateChange();
+            }
+            
+            pendingPowerState = kIOAudioDeviceActive;
+            
+            initiatePowerStateChange();
+        } else if (getPendingPowerState () != kIOAudioDeviceSleep) {
+			// Make sure that when the idle timer fires that we won't go to sleep.
+            pendingPowerState = kIOAudioDeviceActive;
+		}
+    }
+}
+
+void IOAudioDevice::audioEngineStopped()
+{
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::audioEngineStopped() - numRunningAudioEngines = %ld\n", this, numRunningAudioEngines - 1);
+#endif
+
+    numRunningAudioEngines--;
+    
+    if (numRunningAudioEngines == 0) {	// Last audio engine stopping - need to be idle
+        if (getPowerState() == kIOAudioDeviceActive) {	// Go idle
+            if (asyncPowerStateChangeInProgress) {	// Sleep if there is a transition in progress
+                waitForPendingPowerStateChange();
+            }
+            
+            pendingPowerState = kIOAudioDeviceIdle;
+
+			scheduleIdleAudioSleep();
+        }
+    }
+}
+
+IOWorkLoop *IOAudioDevice::getWorkLoop() const
 {
     return workLoop;
 }
 
-IOCommandGate *IOAudioDevice::getCommandGate()
+IOCommandGate *IOAudioDevice::getCommandGate() const
 {
-    if (!commandGate) {
-        IOWorkLoop *wl;
-        
-        wl = getWorkLoop();
-        
-        if (wl) {
-            commandGate = IOCommandGate::commandGate(this);
-            
-            if (commandGate) {
-                wl->addEventSource(commandGate);
-            }
-        }
-    }
-    
     return commandGate;
 }
 
 void IOAudioDevice::setDeviceName(const char *deviceName)
 {
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::setDeviceName(%p)\n", this, deviceName);
+#endif
+
     if (deviceName) {
-        setProperty(IOAUDIODEVICE_NAME_KEY, deviceName);
+        setProperty(kIOAudioDeviceNameKey, deviceName);
+    }
+}
+
+void IOAudioDevice::setDeviceShortName(const char *shortName)
+{
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::setDeviceName(%p)\n", this, shortName);
+#endif
+
+    if (shortName) {
+        setProperty(kIOAudioDeviceShortNameKey, shortName);
     }
 }
 
 void IOAudioDevice::setManufacturerName(const char *manufacturerName)
 {
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::setManufacturerName(%p)\n", this, manufacturerName);
+#endif
+
     if (manufacturerName) {
-        setProperty(IOAUDIODEVICE_MANUFACTURER_NAME_KEY, manufacturerName);
+        setProperty(kIOAudioDeviceManufacturerNameKey, manufacturerName);
     }
 }
 
-bool IOAudioDevice::activateAudioDMAEngine(IOAudioDMAEngine *dmaEngine)
+IOReturn IOAudioDevice::activateAudioEngine(IOAudioEngine *audioEngine)
 {
-    return activateAudioDMAEngine(dmaEngine, true);
+    return activateAudioEngine(audioEngine, true);
 }
 
-bool IOAudioDevice::activateAudioDMAEngine(IOAudioDMAEngine *dmaEngine, bool shouldStartDMAEngine)
+IOReturn IOAudioDevice::activateAudioEngine(IOAudioEngine *audioEngine, bool shouldStartAudioEngine)
 {
-    if (!dmaEngine || !audioDMAEngines) {
-        return false;
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::activateAudioEngine(%p, %d)\n", this, audioEngine, shouldStartAudioEngine);
+#endif
+
+    if (!audioEngine || !audioEngines) {
+        return kIOReturnBadArgument;
     }
 
-    if (!dmaEngine->attach(this)) {
-        return false;
+    if (!audioEngine->attach(this)) {
+        return kIOReturnError;
     }
 
-    if (shouldStartDMAEngine) {
-        if (!dmaEngine->start(this)) {
-            dmaEngine->detach(this);
-            return false;
+    if (shouldStartAudioEngine) {
+        if (!audioEngine->start(this)) {
+            audioEngine->detach(this);
+            return kIOReturnError;
         }
     }
 
-    dmaEngine->deviceStartedDMAEngine = shouldStartDMAEngine;
+    audioEngine->deviceStartedAudioEngine = shouldStartAudioEngine;
 
-    audioDMAEngines->setObject(dmaEngine);
+    audioEngines->setObject(audioEngine);
+    audioEngine->setIndex(audioEngines->getCount()-1);
     
-    dmaEngine->registerService();
-    //dmaEngine->attachToParent(getRegistryRoot(), gIOAudioPlane);
+    audioEngine->registerService();
     
-    return true;
+    return kIOReturnSuccess;
 }
 
-void IOAudioDevice::deactivateAudioDMAEngine(IOAudioDMAEngine *dmaEngine)
+void IOAudioDevice::deactivateAllAudioEngines()
 {
-    if (!dmaEngine || !audioDMAEngines) {
+    OSCollectionIterator *engineIterator;
+    
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::deactivateAllAudioEngines()\n", this);
+#endif
+
+    if (!audioEngines) {
         return;
     }
 
-    dmaEngine->retain();
-    
-    audioDMAEngines->removeObject(dmaEngine);
-
-    // Need to do better than this
-    dmaEngine->stopDMAEngine();
-    
-    if (!isInactive()) {
-        dmaEngine->terminate();
-    }
-    
-    dmaEngine->detachAll(gIOAudioPlane);
+    engineIterator = OSCollectionIterator::withCollection(audioEngines);
+    if (engineIterator) {
+        IOAudioEngine *audioEngine;
         
-    dmaEngine->release();
-}
-
-void IOAudioDevice::deactivateAudioDMAEngines()
-{
-    IOAudioDMAEngine *dmaEngine;
-    
-    if (!audioDMAEngines) {
-        return;
+        while (audioEngine = OSDynamicCast(IOAudioEngine, engineIterator->getNextObject())) {
+            audioEngine->stopAudioEngine();
+            if (!isInactive()) {
+                audioEngine->terminate();
+            }
+        }
+        engineIterator->release();
     }
 
-    while (dmaEngine = OSDynamicCast(IOAudioDMAEngine, audioDMAEngines->getAnyObject())) {
-        deactivateAudioDMAEngine(dmaEngine);
-    }
+    audioEngines->flushCollection();
 }
 
-bool IOAudioDevice::attachAudioPort(IOAudioPort *port, IORegistryEntry *parent, IORegistryEntry *child)
+IOReturn IOAudioDevice::attachAudioPort(IOAudioPort *port, IORegistryEntry *parent, IORegistryEntry *child)
 {
     if (!port || !audioPorts) {
-        return false;
+        return kIOReturnBadArgument;
     }
 
     if (!port->attach(this)) {
-        return false;
+        return kIOReturnError;
     }
 
     if (!port->start(this)) {
         port->detach(this);
-        return false;
+        return kIOReturnError;
     }
 
     audioPorts->setObject(port);
@@ -535,28 +841,10 @@ bool IOAudioDevice::attachAudioPort(IOAudioPort *port, IORegistryEntry *parent,
         child->attachToParent(port, gIOAudioPlane);
     }
 
-    if (port->audioControls) {
-        OSCollectionIterator *iterator;
-
-        iterator = OSCollectionIterator::withCollection(port->audioControls);
-        if (iterator) {
-            OSObject *control;
-
-            while (control = iterator->getNextObject()) {
-                if ((OSDynamicCast(IOAudioLevelControl, control) && ((IOAudioLevelControl *)control)->isMaster()) ||
-                    (OSDynamicCast(IOAudioMuteControl, control) && ((IOAudioMuteControl *)control)->isMaster())) {
-                    addMasterControl((IOAudioControl *)control);
-                }
-            }
-
-            iterator->release();
-        }
-    }
-
-    return true;
+    return kIOReturnSuccess;
 }
 
-void IOAudioDevice::deactivateAudioPorts()
+void IOAudioDevice::detachAllAudioPorts()
 {
     OSCollectionIterator *iterator;
     
@@ -584,6 +872,10 @@ void IOAudioDevice::deactivateAudioPorts()
 
 void IOAudioDevice::flushAudioControls()
 {
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::flushAudioControls()\n", this);
+#endif
+
     if (audioPorts) {
         OSCollectionIterator *portIterator;
 
@@ -612,16 +904,37 @@ void IOAudioDevice::flushAudioControls()
             portIterator->release();
         }
     }
-}
-
-IOReturn IOAudioDevice::performAudioControlValueChange(IOAudioControl *control, UInt32 value)
-{
-    return kIOReturnSuccess;
-}
-
-IOReturn IOAudioDevice::performFormatChange(IOAudioStream *audioStream, const IOAudioStreamFormat *newFormat, const IOAudioSampleRate *newSampleRate)
-{
-    return kIOReturnSuccess;
+    
+    // This code will flush controls attached to an IOAudioPort and a default on a audio engine
+    // more than once
+    // We need to update this to create a single master list of controls and use that to flush
+    // each only once
+    if (audioEngines) {
+        OSCollectionIterator *audioEngineIterator;
+        
+        audioEngineIterator = OSCollectionIterator::withCollection(audioEngines);
+        if (audioEngineIterator) {
+            IOAudioEngine *audioEngine;
+            
+            while (audioEngine = (IOAudioEngine *)audioEngineIterator->getNextObject()) {
+                if (audioEngine->defaultAudioControls) {
+                    OSCollectionIterator *controlIterator;
+                    
+                    controlIterator = OSCollectionIterator::withCollection(audioEngine->defaultAudioControls);
+                    if (controlIterator) {
+                        IOAudioControl *audioControl;
+                        
+                        while (audioControl = (IOAudioControl *)controlIterator->getNextObject()) {
+                            audioControl->flushValue();
+                        }
+                        controlIterator->release();
+                    }
+                }
+            }
+            
+            audioEngineIterator->release();
+        }
+    }
 }
 
 IOReturn IOAudioDevice::addTimerEvent(OSObject *target, TimerEvent event, AbsoluteTime interval)
@@ -747,21 +1060,23 @@ void IOAudioDevice::removeTimerEvent(OSObject *target)
 
             iterator = OSCollectionIterator::withCollection(timerEvents);
             
-            obj = (OSSymbol *)iterator->getNextObject();
-            timerEvent = (IOAudioTimerEvent *)timerEvents->getObject(obj);
-
-            if (timerEvent) {
-                minimumInterval = timerEvent->interval;
-
-                while ((obj = (OSSymbol *)iterator->getNextObject()) && (timerEvent = (IOAudioTimerEvent *)timerEvents->getObject(obj))) {
-                    if (CMP_ABSOLUTETIME(&timerEvent->interval, &minimumInterval) < 0) {
-                        minimumInterval = timerEvent->interval;
+            if (iterator) {
+                obj = (OSSymbol *)iterator->getNextObject();
+                timerEvent = (IOAudioTimerEvent *)timerEvents->getObject(obj);
+    
+                if (timerEvent) {
+                    minimumInterval = timerEvent->interval;
+    
+                    while ((obj = (OSSymbol *)iterator->getNextObject()) && (timerEvent = (IOAudioTimerEvent *)timerEvents->getObject(obj))) {
+                        if (CMP_ABSOLUTETIME(&timerEvent->interval, &minimumInterval) < 0) {
+                            minimumInterval = timerEvent->interval;
+                        }
                     }
                 }
+    
+                iterator->release();
             }
 
-            iterator->release();
-
             assert(timerEventSource);
 
             nextTimerFire = previousTimerFire;
@@ -793,8 +1108,12 @@ void IOAudioDevice::removeTimerEvent(OSObject *target)
     }
 }
 
-void IOAudioDevice::clearTimerEvents()
+void IOAudioDevice::removeAllTimerEvents()
 {
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioDevice[%p]::removeAllTimerEvents()\n", this);
+#endif
+
     if (timerEventSource) {
         timerEventSource->cancelTimeout();
     }
@@ -829,7 +1148,7 @@ void IOAudioDevice::dispatchTimerEvents(bool force)
         IOLog("IOAudioDevice::dispatchTimerEvents() - woke up %lums after last fire - now = {%ld,%lu} - previousFire = {%ld,%lu}\n", (UInt32)(nanos / 1000000), now.hi, now.lo, previousTimerFire.hi, previousTimerFire.lo);
 #endif
 
-        if (force || !wakingFromSleep) {
+        if (force || (getPowerState() != kIOAudioDeviceSleep)) {
             OSIterator *iterator;
             OSSymbol *target;
             AbsoluteTime nextTimerFire, currentInterval;
@@ -844,7 +1163,7 @@ void IOAudioDevice::dispatchTimerEvents(bool force)
                 while (target = (OSSymbol *)iterator->getNextObject()) {
                     IOAudioTimerEvent *timerEvent;
                     timerEvent = (IOAudioTimerEvent *)timerEvents->getObject(target);
-                    
+        
                     if (timerEvent) {
                         (*timerEvent->event)(timerEvent->target, this);
                     }
@@ -884,83 +1203,3 @@ void IOAudioDevice::dispatchTimerEvents(bool force)
     }
 }
 
-void IOAudioDevice::addMasterControl(IOAudioControl *masterControl)
-{
-    if (!masterControls) {
-        masterControls = OSSet::withCapacity(1);
-    }
-
-    masterControls->setObject(masterControl);
-}
-
-void IOAudioDevice::setMasterVolumeLeft(UInt16 newMasterVolumeLeft)
-{
-    if (masterControls) {
-        OSCollectionIterator *iterator;
-
-        iterator = OSCollectionIterator::withCollection(masterControls);
-        if (iterator) {
-            OSObject *control;
-            while (control = iterator->getNextObject()) {
-                IOAudioLevelControl *levelControl;
-
-                levelControl = OSDynamicCast(IOAudioLevelControl, control);
-                if (levelControl && (levelControl->getChannelID() == IOAUDIOCONTROL_CHANNEL_ID_DEFAULT_LEFT)) {
-                    UInt32 minValue, maxValue;
-                    minValue = levelControl->getMinValue();
-                    maxValue = levelControl->getMaxValue();
-
-                    levelControl->setValue(((newMasterVolumeLeft - MASTER_VOLUME_MIN) * (maxValue - minValue) / (MASTER_VOLUME_MAX - MASTER_VOLUME_MIN)) + minValue);
-                }
-            }
-            iterator->release();
-        }
-    }
-}
-
-void IOAudioDevice::setMasterVolumeRight(UInt16 newMasterVolumeRight)
-{
-    if (masterControls) {
-        OSCollectionIterator *iterator;
-
-        iterator = OSCollectionIterator::withCollection(masterControls);
-        if (iterator) {
-            OSObject *control;
-            while (control = iterator->getNextObject()) {
-                IOAudioLevelControl *levelControl;
-
-                levelControl = OSDynamicCast(IOAudioLevelControl, control);
-                if (levelControl && (levelControl->getChannelID() == IOAUDIOCONTROL_CHANNEL_ID_DEFAULT_RIGHT)) {
-                    UInt32 minValue, maxValue;
-                    minValue = levelControl->getMinValue();
-                    maxValue = levelControl->getMaxValue();
-
-                    levelControl->setValue(((newMasterVolumeRight - MASTER_VOLUME_MIN) * (maxValue - minValue) / (MASTER_VOLUME_MAX - MASTER_VOLUME_MIN)) + minValue);
-                }
-            }
-            iterator->release();
-        }
-    }
-}
-
-void IOAudioDevice::setMasterMute(bool newMasterMute)
-{
-    if (masterControls) {
-        OSCollectionIterator *iterator;
-
-        iterator = OSCollectionIterator::withCollection(masterControls);
-        if (iterator) {
-            OSObject *control;
-            while (control = iterator->getNextObject()) {
-                IOAudioMuteControl *muteControl;
-
-                muteControl = OSDynamicCast(IOAudioMuteControl, control);
-                if (muteControl) {
-                    muteControl->setValue(newMasterMute ? 1 : 0);
-                }
-            }
-            iterator->release();
-        }
-    }
-}
-
